@@ -637,7 +637,10 @@ namespace IPod {
         PlaylistData = 50,
         PlaylistRules = 51,
         LibraryIndex = 52,
-        Misc = 100
+        Misc = 100,
+        AlbumListAlbum = 200,
+        AlbumListArtist = 201,
+        AlbumListPodcastUrl = 203
     }
 
     internal enum IndexType {
@@ -681,13 +684,14 @@ namespace IPod {
             
             Type = (DetailType) ToInt32 (body, 0);
 
-            if ((int) Type > 50 && Type != DetailType.Misc && Type != DetailType.LibraryIndex)
+            if ((int) Type > 50 && Type != DetailType.Misc && Type != DetailType.LibraryIndex &&
+                Type != DetailType.AlbumListArtist && Type != DetailType.AlbumListAlbum && Type != DetailType.AlbumListPodcastUrl)
                 throw new DatabaseReadException ("Unsupported detail type: " + Type);
 
             unknownOne = ToInt32 (body, 4);
             unknownTwo = ToInt32 (body, 8);
             
-            if ((int) Type < 50) {
+            if ((int) Type < 50 || Type == DetailType.AlbumListArtist || Type == DetailType.AlbumListAlbum || Type == DetailType.AlbumListPodcastUrl) {
                 if (Type == DetailType.PodcastUrl ||
                     Type == DetailType.PodcastUrl2) {
 
@@ -704,7 +708,7 @@ namespace IPod {
                     int strlen = 0;
                     //int strenc = 0;
             
-                    if ((int) Type < 50) {
+                    if ((int) Type < 50 || Type == DetailType.AlbumListArtist || Type == DetailType.AlbumListAlbum || Type == DetailType.AlbumListPodcastUrl) {
                         // 'string' mhods       
                         strlen = ToInt32 (body, 16);
                         //strenc = ToInt32 (body, 20); // 0 == UTF16, 1 == UTF8
@@ -754,7 +758,8 @@ namespace IPod {
 
             byte[] valbytes = null;
 
-            if ((int) Type < 50) {
+            if ((int) Type < 50 || Type == DetailType.AlbumListArtist || Type == DetailType.AlbumListAlbum ||
+                Type == DetailType.AlbumListPodcastUrl) {
                 if (Type == DetailType.PodcastUrl || Type == DetailType.PodcastUrl2) {
                     valbytes = Encoding.UTF8.GetBytes (Value);
                     writer.Write (24 + valbytes.Length);
@@ -781,7 +786,8 @@ namespace IPod {
             writer.Write (unknownOne);
             writer.Write (unknownTwo);
 
-            if ((int) Type < 50) {
+            if ((int) Type < 50 || Type == DetailType.AlbumListArtist || Type == DetailType.AlbumListAlbum ||
+                Type == DetailType.AlbumListPodcastUrl) {
                 if (Type == DetailType.PodcastUrl || Type == DetailType.PodcastUrl2 ||
                     Type == DetailType.ChapterData) {
                     writer.Write (valbytes);
@@ -1244,10 +1250,138 @@ namespace IPod {
         }
     }
 
+    // should be AlbumListRecord, but that exists in PhotoDatabase already, sigh
+    internal class TrackAlbumListRecord : TrackDbRecord {
+
+        private List<TrackAlbumItemRecord> albums = new List<TrackAlbumItemRecord> ();
+
+        public IList<TrackAlbumItemRecord> Albums {
+            get { return new ReadOnlyCollection<TrackAlbumItemRecord> (albums); }
+        }
+
+        public TrackAlbumListRecord (bool isbe) : base (isbe) {
+            this.Name = "mhla";
+        }
+
+        public void Clear () {
+            albums.Clear ();
+        }
+
+        public void Remove (TrackAlbumItemRecord item) {
+            albums.Remove (item);
+        }
+
+        public void Add (TrackAlbumItemRecord item) {
+            albums.Add (item);
+        }
+        
+        public override void Read (DatabaseRecord db, BinaryReader reader) {
+
+            base.Read (db, reader);
+            
+            reader.ReadBytes (this.HeaderOne - 12);
+
+            int albumCount = this.HeaderTwo;
+
+            albums.Clear ();
+
+            for (int i = 0; i < albumCount; i++) {
+                TrackAlbumItemRecord rec = new TrackAlbumItemRecord (IsBE);
+                rec.Read (db, reader);
+                albums.Add (rec);
+            }
+        }
+
+        public override void Save (DatabaseRecord db, BinaryWriter writer) {
+
+            MemoryStream stream = new MemoryStream ();
+            BinaryWriter childWriter = new EndianBinaryWriter (stream, IsBE);
+
+            foreach (TrackAlbumItemRecord rec in albums) {
+                rec.Save (db, childWriter);
+            }
+
+            childWriter.Flush ();
+            byte[] childData = stream.GetBuffer ();
+            int childDataLength = (int) stream.Length;
+            childWriter.Close ();
+
+            WriteName (writer);
+            writer.Write (12 + PadLength);
+            writer.Write (albums.Count);
+            writer.Write (new byte[PadLength]);
+            writer.Write (childData, 0, childDataLength);
+        }
+    }
+
+    internal class TrackAlbumItemRecord : TrackDbRecord {
+
+        private List<DetailRecord> details = new List<DetailRecord> ();
+
+        public IList<DetailRecord> Details {
+            get { return new ReadOnlyCollection<DetailRecord> (details); }
+        }
+
+        public TrackAlbumItemRecord (bool isbe) : base (isbe) {
+            this.Name = "mhia";
+        }
+
+        public void Clear () {
+            details.Clear ();
+        }
+
+        public void Remove (DetailRecord item) {
+            details.Remove (item);
+        }
+
+        public void Add (DetailRecord item) {
+            details.Add (item);
+        }
+        
+        public override void Read (DatabaseRecord db, BinaryReader reader) {
+
+            base.Read (db, reader);
+            
+            byte[] body = reader.ReadBytes (this.HeaderOne - 12);
+
+            int detailCount = ToInt32 (body, 0);
+
+            details.Clear ();
+
+            for (int i = 0; i < detailCount; i++) {
+                DetailRecord detail = new DetailRecord (IsBE);
+                detail.Read (db, reader);
+                details.Add (detail);
+            }
+        }
+
+        public override void Save (DatabaseRecord db, BinaryWriter writer) {
+
+            MemoryStream stream = new MemoryStream ();
+            BinaryWriter childWriter = new EndianBinaryWriter (stream, IsBE);
+
+            foreach (DetailRecord detail in details) {
+                detail.Save (db, childWriter);
+            }
+
+            childWriter.Flush ();
+            byte[] childData = stream.GetBuffer ();
+            int childDataLength = (int) stream.Length;
+            childWriter.Close ();
+
+            WriteName (writer);
+            writer.Write (16);
+            writer.Write (16 + childDataLength);
+            writer.Write (details.Count);
+            writer.Write (childData, 0, childDataLength);
+        }
+    }
+
     internal enum DataSetIndex {
         Library = 1,
         Playlist = 2,
-        PlaylistDuplicate = 3
+        PlaylistDuplicate = 3,
+        AlbumList = 4
     }
 
     internal class DataSetRecord : TrackDbRecord {
@@ -1256,6 +1390,7 @@ namespace IPod {
 
         public TrackListRecord TrackList;
         public PlaylistListRecord PlaylistList;
+        public TrackAlbumListRecord AlbumList;
 
         public PlaylistRecord Library {
             get {
@@ -1298,6 +1433,10 @@ namespace IPod {
                 this.PlaylistList = new PlaylistListRecord (IsBE);
                 this.PlaylistList.Read (db, reader);
                 break;
+            case DataSetIndex.AlbumList:
+                this.AlbumList = new TrackAlbumListRecord (IsBE);
+                this.AlbumList.Read (db, reader);
+                break;
             default:
                 throw new DatabaseReadException ("Can't handle dataset index: " + Index);
             }
@@ -1319,6 +1458,9 @@ namespace IPod {
             case DataSetIndex.PlaylistDuplicate:
                 PlaylistList.Save (db, childWriter);
                 break;
+            case DataSetIndex.AlbumList:
+                AlbumList.Save (db, childWriter);
+                break;
             default:
                 throw new DatabaseReadException ("Can't handle DataSet record index: " + Index);
             }
@@ -1339,7 +1481,7 @@ namespace IPod {
 
     internal class DatabaseRecord : TrackDbRecord {
 
-        private const int MaxSupportedVersion = 19;
+        private const int MaxSupportedVersion = 20;
         private const int TrackIdStart = 1000;
 
         private int unknownOne = 1;
@@ -1421,7 +1563,6 @@ namespace IPod {
         }
 
         public override void Save (DatabaseRecord db, BinaryWriter writer) {
-
             ReassignTrackIds ();
             
             MemoryStream stream = new MemoryStream ();
@@ -1611,13 +1752,17 @@ namespace IPod {
         internal TrackDatabase (Device device, bool createFresh) {
             this.device = device;
             
-            if(createFresh && File.Exists(TrackDbPath)) {
+            if(createFresh && File.Exists(TrackDbPath) && device.CanWrite) {
                 File.Copy (TrackDbPath, TrackDbBackupPath, true);
             }
 
             Reload (createFresh);
 
-            artdb = new PhotoDatabase (device, false, createFresh);
+            try {
+                artdb = new PhotoDatabase (device, false, createFresh);
+            } catch {
+                artdb = null;
+            }
         }
         
         
@@ -1811,7 +1956,7 @@ namespace IPod {
 
                 uniqueName += ext;
             } while(File.Exists(basePath + uniqueName));
-				
+
             return uniqueName.Replace("/", ":");
         }
 
@@ -1856,9 +2001,51 @@ namespace IPod {
             }
         }
 
+        private void CreateAlbumList () {
+            DataSetRecord ds = dbrec[DataSetIndex.AlbumList];
+
+            // FIXME: figure out when we should create one of these if it doesn't exit
+            if (ds == null)
+                return;
+
+            TrackAlbumListRecord albumList = ds.AlbumList;
+
+            albumList.Clear ();
+            
+            List<string> albums = new List<string> ();
+            
+            foreach (Track track in tracks) {
+                string key = track.Album + "|" + track.Artist;
+                if (!albums.Contains (key)) {
+                    albums.Add (key);
+                }
+            }
+            
+            foreach (string album in albums) {
+                TrackAlbumItemRecord item = new TrackAlbumItemRecord (dbrec.IsBE);
+                
+                string[] splitAlbum = album.Split(new char[] {'|'}, 2);
+                if (splitAlbum.Length != 2)
+                    continue;
+                
+                DetailRecord albumDetail = new DetailRecord (dbrec.IsBE);
+                albumDetail.Type = DetailType.AlbumListAlbum;
+                albumDetail.Value = splitAlbum[0];
+                
+                DetailRecord artistDetail = new DetailRecord (dbrec.IsBE);
+                artistDetail.Type = DetailType.AlbumListArtist;
+                artistDetail.Value = splitAlbum[1];
+                
+                item.Add (albumDetail);
+                item.Add (artistDetail);
+                albumList.Add (item);
+            }
+        }
+
         public void Save () {
 
             CheckFreeSpace ();
+            CreateAlbumList ();
 
             // make sure all the new tracks have file names, and that they exist
             foreach (Track track in tracksToAdd) {
@@ -1874,7 +2061,7 @@ namespace IPod {
                 SaveStarted (this, new EventArgs ());
 
             // Back up the current track db
-            if (File.Exists (TrackDbPath))
+            if (File.Exists (TrackDbPath) && device.CanWrite)
                 File.Copy (TrackDbPath, TrackDbBackupPath, true);
             
             try {
@@ -1904,7 +2091,9 @@ namespace IPod {
                 }
 
                 // Save artwork database
-                artdb.Save ();
+                if (artdb != null) {
+                    artdb.Save ();
+                }
 
                 // Save the shuffle tracks db (will only create if device is shuffle);
                 try {
@@ -1938,7 +2127,7 @@ namespace IPod {
                 Mono.Unix.Native.Syscall.sync ();
             } catch (Exception e) {
                 // rollback the track db
-                if (File.Exists (TrackDbBackupPath))
+                if (File.Exists (TrackDbBackupPath) && device.CanWrite)
                     File.Copy (TrackDbBackupPath, TrackDbPath, true);
 
                 throw new DatabaseWriteException (e, "Failed to save database");
@@ -2020,11 +2209,13 @@ namespace IPod {
                 dbrec[DataSetIndex.Playlist].Library.RemoveTrack (track.Record.Id);
 
                 // remove from cover art db
-                Photo artPhoto = artdb.LookupPhotoByTrackId (track.Record.DatabaseId);
-                if (artPhoto != null) {
-                    artdb.RemovePhoto (artPhoto);
+                if (artdb != null) {
+                    Photo artPhoto = artdb.LookupPhotoByTrackId (track.Record.DatabaseId);
+                    if (artPhoto != null) {
+                        artdb.RemovePhoto (artPhoto);
+                    }
                 }
-                    
+
                 // remove from the "normal" playlists
                 foreach (Playlist list in playlists) {
                     list.RemoveTrack (track);
